@@ -1,4 +1,5 @@
 from typing import Optional, Union, List, Dict, Any, Callable, Awaitable
+
 from ..objects.animation import Animation
 from ..objects.audio import Audio
 from ..objects.callbackquery import CallbackQuery
@@ -41,11 +42,12 @@ from ..objects.webappinfo import WebAppInfo
 from ..objects.utils import *
 from ..objects.enums import UpdatesTypes, ChatAction, ChatType, ChatPermissions
 from ..objects.peerdata import PeerData
+from ..objects.transaction import Transaction
 from ..filters import Filters
 from ..StateMachine import StateMachine
 from ..exceptions import NotFoundException, InvalidTokenException, PyroBaleException, ForbiddenException
 import time
-from enum import Enum
+from enum import Enum, member
 import asyncio
 from bs4 import BeautifulSoup
 from json import loads, JSONDecodeError, dumps
@@ -79,6 +81,7 @@ class Client:
         self.check_defined_message = True
         self.defined_messages = {}
 
+    @smart_method
     async def ping(self, round_it = False) -> float:
         """
         Ping the bot to see if the bot is alive.
@@ -100,6 +103,7 @@ class Client:
             except Exception as e:
                 raise e
 
+    @smart_method
     async def get_updates(
             self,
             offset: Optional[int] = None,
@@ -127,6 +131,7 @@ class Client:
                     raise InvalidTokenException("Forbidden 403 : --ENTERED TOKEN IS NOT VALID--")
         return []
 
+    @smart_method
     async def set_webhook(self, url: str) -> bool:
         """Set the webhook for the bot.
 
@@ -139,6 +144,7 @@ class Client:
         data = await make_post(self.requests_base + "/setWebhook", data={"url": url})
         return data.get("ok", False)
 
+    @smart_method
     async def get_webhook_info(self) -> Dict:
         """Get the webhook information for the bot.
 
@@ -148,6 +154,7 @@ class Client:
         data = await make_get(self.requests_base + "/getWebhookInfo")
         return data.get("result", {})
 
+    @smart_method
     async def get_me(self) -> User:
         """Get information about the bot.
 
@@ -159,6 +166,7 @@ class Client:
             raise InvalidTokenException("Token is invalid")
         return User(**data["result"])
 
+    @smart_method
     async def logout(self) -> bool:
         """Log out the bot.
 
@@ -168,6 +176,7 @@ class Client:
         data = await make_get(self.requests_base + "/logOut")
         return data.get("ok", False)
 
+    @smart_method
     async def close(self) -> bool:
         """Close the bot.
 
@@ -177,6 +186,7 @@ class Client:
         data = await make_get(self.requests_base + "/close")
         return data.get("ok", False)
 
+    @smart_method
     async def send_message(
             self,
             chat_id: int,
@@ -206,6 +216,7 @@ class Client:
         )
         return Message(**pythonize(data.get("result")))
 
+    @smart_method
     async def delete_message(
             self,
             chat_id: int,
@@ -229,6 +240,7 @@ class Client:
         )
         return True
 
+    @smart_method
     async def forward_message(
             self, chat_id: int, from_chat_id: int, message_id: int
     ) -> Message:
@@ -252,6 +264,7 @@ class Client:
         )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def copy_message(
             self, chat_id: int, from_chat_id: int, message_id: int
     ) -> Message:
@@ -275,6 +288,7 @@ class Client:
         )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_photo(
         self,
         chat_id: Union[int, str],
@@ -296,21 +310,21 @@ class Client:
         if isinstance(photo, InputFile):
             form = aiohttp.FormData()
             form.add_field("chat_id", str(chat_id))
-            
-            
+
+
             if photo.file_name:
                 form.add_field("photo", photo.file_input, filename=photo.file_name)
             else:
                 form.add_field("photo", photo.file_input, filename="photo.png")
-            
-            
+
+
             if caption:
                 form.add_field("caption", caption)
             if reply_to_message_id:
                 form.add_field("reply_to_message_id", str(reply_to_message_id))
             if reply_markup:
                 form.add_field("reply_markup", dumps(reply_markup.to_dict()))
-            
+
             url = self.requests_base + handler
             data = await make_via_multipart(url, form)
         else:
@@ -326,6 +340,7 @@ class Client:
             )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_audio(
         self,
         chat_id: Union[int, str],
@@ -347,21 +362,21 @@ class Client:
         if isinstance(audio, InputFile):
             form = aiohttp.FormData()
             form.add_field("chat_id", str(chat_id))
-            
-            
+
+
             if audio.file_name:
                 form.add_field("audio", audio.file_input, filename=audio.file_name)
             else:
                 form.add_field("audio", audio.file_input, filename="audio.mp3")
-            
-            
+
+
             if caption:
                 form.add_field("caption", caption)
             if reply_to_message_id:
                 form.add_field("reply_to_message_id", str(reply_to_message_id))
             if reply_markup:
                 form.add_field("reply_markup", dumps(reply_markup.to_dict()))
-            
+
             url = self.requests_base + handler
             data = await make_via_multipart(url, form)
         else:
@@ -377,6 +392,7 @@ class Client:
             )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_document(
             self,
             chat_id: Union[int, str],
@@ -398,21 +414,21 @@ class Client:
         if isinstance(document, InputFile):
             form = aiohttp.FormData()
             form.add_field("chat_id", str(chat_id))
-            
-            
+
+
             if document.file_name:
                 form.add_field("document", document.file_input, filename=document.file_name)
             else:
                 form.add_field("document", document.file_input, filename="document.pdf")
-            
-            
+
+
             if caption:
                 form.add_field("caption", caption)
             if reply_to_message_id:
                 form.add_field("reply_to_message_id", str(reply_to_message_id))
             if reply_markup:
                 form.add_field("reply_markup", dumps(reply_markup.to_dict()))
-            
+
             url = self.requests_base + handler
             data = await make_via_multipart(url, form)
         else:
@@ -428,6 +444,7 @@ class Client:
             )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_video(
             self,
             chat_id: Union[int, str],
@@ -449,21 +466,21 @@ class Client:
         if isinstance(video, InputFile):
             form = aiohttp.FormData()
             form.add_field("chat_id", str(chat_id))
-            
-            
+
+
             if video.file_name:
                 form.add_field("video", video.file_input, filename=video.file_name)
             else:
                 form.add_field("video", video.file_input, filename="video.mp4")
-            
-            
+
+
             if caption:
                 form.add_field("caption", caption)
             if reply_to_message_id:
                 form.add_field("reply_to_message_id", str(reply_to_message_id))
             if reply_markup:
                 form.add_field("reply_markup", dumps(reply_markup.to_dict()))
-            
+
             url = self.requests_base + handler
             data = await make_via_multipart(url, form)
         else:
@@ -479,6 +496,7 @@ class Client:
             )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_animation(
             self,
             chat_id: Union[int, str],
@@ -500,21 +518,21 @@ class Client:
         if isinstance(animation, InputFile):
             form = aiohttp.FormData()
             form.add_field("chat_id", str(chat_id))
-            
-            
+
+
             if animation.file_name:
                 form.add_field("animation", animation.file_input, filename=animation.file_name)
             else:
                 form.add_field("animation", animation.file_input, filename="animation.gif")
-            
-            
+
+
             if caption:
                 form.add_field("caption", caption)
             if reply_to_message_id:
                 form.add_field("reply_to_message_id", str(reply_to_message_id))
             if reply_markup:
                 form.add_field("reply_markup", dumps(reply_markup.to_dict()))
-            
+
             url = self.requests_base + handler
             data = await make_via_multipart(url, form)
         else:
@@ -530,6 +548,7 @@ class Client:
             )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_voice(
             self,
             chat_id: Union[int, str],
@@ -551,21 +570,21 @@ class Client:
         if isinstance(voice, InputFile):
             form = aiohttp.FormData()
             form.add_field("chat_id", str(chat_id))
-            
-            
+
+
             if voice.file_name:
                 form.add_field("voice", voice.file_input, filename=voice.file_name)
             else:
                 form.add_field("voice", voice.file_input, filename="voice.ogg")
-            
-            
+
+
             if caption:
                 form.add_field("caption", caption)
             if reply_to_message_id:
                 form.add_field("reply_to_message_id", str(reply_to_message_id))
             if reply_markup:
                 form.add_field("reply_markup", dumps(reply_markup.to_dict()))
-            
+
             url = self.requests_base + handler
             data = await make_via_multipart(url, form)
         else:
@@ -581,6 +600,7 @@ class Client:
             )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_media_group(
             self,
             chat_id: int,
@@ -610,6 +630,7 @@ class Client:
         )
         return [Message(**pythonize(msg)) for msg in data["result"]]
 
+    @smart_method
     async def send_location(
             self,
             chat_id: int,
@@ -645,6 +666,7 @@ class Client:
         )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_contact(
             self,
             chat_id: int,
@@ -680,6 +702,7 @@ class Client:
         )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def send_invoice(
             self,
             chat_id: Union[str, int],
@@ -721,7 +744,17 @@ class Client:
             },
         )
         return Message(**pythonize(data["result"]))
+    
+    @smart_method
+    async def inquire_transaction(self, transaction_id: str) -> Transaction:
+        
+        data = await make_post(
+            self.requests_base + "/inquireTransaction",
+            data={"transaction_id": transaction_id} 
+        )
+        return Transaction(**pythonize(data['result']))
 
+    @smart_method
     async def get_file(self, file_id: str) -> File:
         """Get a file from the Bale servers.
 
@@ -736,6 +769,7 @@ class Client:
         )
         return File(**pythonize(data["result"]))
 
+    @smart_method
     async def answer_callback_query(
             self,
             callback_query_id: str,
@@ -762,6 +796,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def ban_chat_member(self, chat_id: int, user_id: int) -> bool:
         """Ban a user from a chat.
 
@@ -780,7 +815,49 @@ class Client:
             return data.get("ok", False)
         except AttributeError:
             raise ForbiddenException("You cannot ban this member!")
+    
+    @smart_method
+    async def restrict_chat_member(self,
+                                   chat_id: int,
+                                   user_id: int,
+                                   can_send_messages: Union[bool,None] = None,
+                                   can_send_media_messages: Union[bool,None] = None,
+                                   can_send_other_messages: Union[bool,None] = None,
+                                   can_add_web_page_previews: Union[bool,None] = None,
+                                   until_date: Union[int,None] = None
+    ) -> bool:
+        """Restricts a user from a chat.
 
+        Args:
+            chat_id (int): The chat to ban.
+            user_id (int): The user to ban.
+            can_send_messages (Union[bool,None]): Default is None
+            can_send_media_messages (Union[bool,None]): Default is None
+            can_send_other_messages (Union[bool,None]): Default is None
+            can_add_web_page_previews (Union[bool,None]): Default is None
+            until_date: (Union[int,None]) Default is None
+
+        Returns:
+            bool: Whether the ban was successful.
+        """
+        data = await make_post(
+            self.requests_base + "/restrictChatMember",
+            data={"chat_id": chat_id, "user_id": user_id,
+            "permissions": {
+                "can_send_messages": can_send_messages,
+                "can_send_media_messages": can_send_media_messages,
+                "can_send_other_messages": can_send_other_messages,
+                "can_add_web_page_previews": can_add_web_page_previews
+            },
+            "until_date": until_date
+            },
+        )
+        try:
+            return data.get("ok", False)
+        except AttributeError:
+            raise ForbiddenException("You cannot restrict this member!")
+
+    @smart_method
     async def unban_chat_member(self, chat_id: int, user_id: int) -> bool:
         """Unban a user from a chat.
 
@@ -797,6 +874,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def kick_chat_member(self, chat_id: int, user_id: int) -> bool:
         """kick a user from a specified chat
 
@@ -814,6 +892,29 @@ class Client:
         except:
             return False
 
+    @smart_method
+    async def get_chat_administrators(self, chat_id: int) -> List[ChatMember]:
+        """Gets a list of administrators of a specified chat.
+
+        Args:
+            chat_id: Chat id of the chat to get administrators for.
+
+        Returns:
+            A list of administrators.
+        """
+
+        data = await make_post(
+            self.requests_base + "/getChatAdministrators",
+            data={"chat_id": chat_id},
+        )
+
+        if not data:
+            raise ForbiddenException("You cannot get administrators for this chat!")
+
+        res = data["result"]
+        return [ChatMember(**member) for member in res]
+
+    @smart_method
     async def get_chat_member(self, chat_id: int, user_id: int) -> ChatMember:
         """Get a chat member.
 
@@ -838,6 +939,7 @@ class Client:
 
         return ChatMember(**pythonize(data))
 
+    @smart_method
     async def is_user_admin(self, chat_id: int, user_id: int) -> bool:
         """Checks if a user is admin in a chat
 
@@ -858,6 +960,7 @@ class Client:
         else:
             return False
 
+    @smart_method
     async def user_has_permissions(self, chat_id: int, user_id: int, permissions: ChatPermissions) -> bool:
         """checks if a user has a specified permission
 
@@ -875,6 +978,7 @@ class Client:
         else:
             return False
 
+    @smart_method
     async def promote_chat_member(
             self,
             chat_id: int,
@@ -922,6 +1026,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def set_chat_photo(self, chat_id: int, photo: InputFile) -> bool:
         """Set a new profile photo for the chat.
 
@@ -938,6 +1043,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def leave_chat(self, chat_id: int) -> bool:
         """Leave a group, supergroup or channel.
 
@@ -952,6 +1058,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def is_joined(self, user_id: int, chat_id: int) -> bool:
         """Check if a user is joined to a chat.
 
@@ -968,6 +1075,7 @@ class Client:
         )
         return data.get("result", {}).get("status") in ["member", "creator", "administrator"]
 
+    @smart_method
     async def get_chat(self, chat_id: int) -> Chat:
         """Get up to date information about the chat.
 
@@ -987,6 +1095,7 @@ class Client:
         return Chat(**pythonize(data))
 
     @staticmethod
+    @smart_method
     async def get_ble_ir_page(username_or_phone_number: str) -> PeerData:
         """Get BleIR user/group information.
 
@@ -1079,6 +1188,7 @@ class Client:
             username
         )
 
+    @smart_method
     async def get_chat_members_count(self, chat_id: int) -> int:
         """Get the number of members in a chat.
 
@@ -1093,6 +1203,7 @@ class Client:
         )
         return data.get("result", 0)
 
+    @smart_method
     async def pin_chat_message(self, chat_id: int, message_id: int) -> bool:
         """Pin a message in a chat.
 
@@ -1109,6 +1220,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def unpin_chat_message(self, chat_id: int, message_id: int) -> bool:
         """Unpin a message in a chat.
 
@@ -1124,6 +1236,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def unpin_all_chat_messages(self, chat_id: int) -> bool:
         """Unpin all messages in a chat.
 
@@ -1138,6 +1251,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def set_chat_title(self, chat_id: int, title: str) -> bool:
         """Change the title of a chat.
 
@@ -1154,6 +1268,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def set_chat_description(self, chat_id: int, description: str) -> bool:
         """Change the description of a chat.
 
@@ -1170,6 +1285,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def delete_chat_photo(self, chat_id: int) -> bool:
         """Delete a chat photo.
 
@@ -1184,6 +1300,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def edit_message(
             self,
             chat_id: Union[int, str],
@@ -1213,6 +1330,7 @@ class Client:
         )
         return Message(**pythonize(data["result"]))
 
+    @smart_method
     async def create_chat_invite_link(self, chat_id: int) -> InviteLink:
         """Create an additional invite link for a chat.
 
@@ -1230,6 +1348,7 @@ class Client:
         except AttributeError:
             raise ForbiddenException("you cannot access this chat")
 
+    @smart_method
     async def revoke_chat_invite_link(self, chat_id: int, invite_link: str) -> str:
         """Revoke an invite link created by the bot.
 
@@ -1246,6 +1365,7 @@ class Client:
         )
         return data.get("result", "")
 
+    @smart_method
     async def export_chat_invite_link(self, chat_id: int) -> str:
         """Generate a new primary invite link for a chat.
 
@@ -1260,6 +1380,7 @@ class Client:
         )
         return data.get("result", "")
 
+    @smart_method
     async def send_chat_action(self, chat_id: int, action: ChatAction) -> bool:
         """Tell the user that something is happening on the bot's side.
 
@@ -1276,6 +1397,7 @@ class Client:
         )
         return data.get("ok", False)
 
+    @smart_method
     async def wait_for(self, update_type: UpdatesTypes, check=None, timeout: Optional[float] = None):
         """Wait until a specified update
 
@@ -1299,6 +1421,7 @@ class Client:
             self._waiters = [w for w in self._waiters if w[2] != future]
             raise
 
+    @smart_method
     async def process_update(self, update: Dict[str, Any]) -> None:
         """Process a single update and call registered handlers."""
         update_id = update.get("update_id")
@@ -1586,6 +1709,7 @@ class Client:
         """Remove all handlers from the list of handlers."""
         self.handlers = []
 
+    @smart_method
     async def start_polling(self, timeout: int = 30, limit: int = 100) -> None:
         """Start polling updates from the server.
 
@@ -1612,6 +1736,7 @@ class Client:
                 print(f"Error in polling: {e}")
                 await asyncio.sleep(1)
 
+    @smart_method
     async def stop_polling(self) -> None:
         """Stop polling updates."""
         self.running = False
@@ -1629,10 +1754,12 @@ class Client:
         except KeyboardInterrupt:
             print("Bot stopped by user")
 
+    @smart_method
     async def stop(self) -> None:
         """Stop the client."""
         await self.stop_polling()
 
+    @smart_method
     async def handle_webhook_update(self, update_data: Dict[str, Any]) -> None:
         """Process an update received via webhook."""
         await self.process_update(update_data)
\ No newline at end of file
